<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Config;
use Auth;
use Illuminate\Support\Arr;
use PDF;

class SaleController extends Controller
{
    public function saleList()
    {
        $Queries = array();
        $list = DB::table('sales')
            ->join('customers', 'sales.customer_id', '=', 'customers.id')
            ->select('sales.*', 'customers.name as customer_name')->where('sales.branch', Auth::user()->branch)
            ->orderByDesc('sales.id')
            ->paginate(20);
        $net_total = DB::table('sales')->where('branch', Auth::user()->branch)->sum('net_total');
        $net_qty = DB::table('sales')->where('branch', Auth::user()->branch)->sum('net_qty');
        $net_pcs = DB::table('sales')->where('branch', Auth::user()->branch)->sum('net_pcs');
        $net_custom_rate = DB::table('sales')->where('branch', Auth::user()->branch)->sum('net_custom_rate');
        $customers = DB::table('customers')->where('branch', Auth::user()->branch)->get();
        return view('sales.list', array('salelist' => $list, 'queries' => $Queries, 'customers' => $customers, 'net_total' => $net_total, 'net_pcs' => $net_pcs, 'net_qty' => $net_qty,'net_custom_rate' => $net_custom_rate));
    }
    public function newsale()
    {
        $customers = DB::table('customers')->where('status', 1)->where('branch', Auth::user()->branch)->get();
        $invoice_number = DB::table('sales')->max('id') + 1;

        $items = DB::table('items')->where('branch', Auth()->user()->branch)
            ->whereIn('category', [4, 5, 6])
            ->get();
        return view('sales.new', array('customers' => $customers, 'invoice_number' => $invoice_number, 'items' => $items,));
    }
    public function store(Request $request)
    {
        $response = array('success' => false, 'message' => '', 'redirectUrl' => '');
        $sale = DB::table('sales')->where('invoice_number', $request->invoice_number)->first();
        if (!empty($sale)) {
            return response()->json(['success' => false, 'message' => 'Sale Invoice already exits..', 'redirectUrl' => '/sales/list'], 200);
        }


        $validator = Validator::make(
            $request->all(),
            [
                'invoice_number' => 'required',
                'invoice_date' => 'required',
                'customer_id' => 'required|numeric',
                'net_total' => 'required|numeric|min:0|not_in:0',
                'note' => 'required',
                'net_pcs' => 'required|numeric|min:0|not_in:0',
                'net_qty' => 'required|numeric|min:0|not_in:0',
            ],
            [
                'invoice_number.required' => 'The Invoice #  is required.',
                'invoice_date.required' => 'The Invoice Date  is required.',
                'customer_id.required' => 'The Customer   is required.',
                'net_total.required' => 'Net Total   is required.',
                'note.required' => 'Notes   is required.',
                'net_pcs.required' => 'Pcs is required.',
                'net_qty.required' => 'Qty is required.',
            ]
        );
        if ($validator->fails()) {
            //$response['message'] = $validator->messages();
            return response()->json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray()

            ), 422);
        } else {

            $items_detail = array();
            $item_ids = $request->item_id;
            $item_prices = $request->item_price;
            $item_qtys = $request->item_qty;
            $item_pcss = $request->item_pcs;
            $item_custom_rates = $request->item_custom_rate;
            $total_custom_rates = $request->total_custom_rate;
            $amounts = $request->amount;
            $i = 0;
            foreach ($item_ids as $item) {
                $itemid = $item_ids[$i];
                $price = $item_prices[$i];
                $qty = $item_qtys[$i];
                $pcs = $item_pcss[$i];
                $item_custom_rate = $item_custom_rates[$i];
                $total_custom_rate = $total_custom_rates[$i];
                $amount = $amounts[$i];

                if ($amount > 0) {
                    $items_detail[] = array(
                        'item_id' => $itemid,
                        'item_price' => $price,
                        'item_qty' => $qty,
                        'amount' => $amount,
                        'item_custom_rate' => $item_custom_rate,
                        'total_custom_rate' => $total_custom_rate,
                        'item_pcs' => $pcs,
                    );
                }

                $i++;
            }
            /**
             * Insert Double entry
             *Customer A/c Debit
             *Sale A/c  Credit
             */
            $customer = DB::table('customers')->where('id', $request->customer_id)->first();
            $debit = array(
                'voucher_date' => $request->invoice_date,
                'voucher_number' => $request->invoice_number,
                'general_ledger_account_id' => $customer->general_ledger_account_id,
                'note' => $request->note,
                'debit' => $request->net_total,
                'credit' => 0,
                'branch' => Auth::user()->branch,
                'created_at' => date('Y-m-d H:i:s'),
            );
            $this->insertDoubleEntry($debit);
            if(isset($request->cash_entry)){
                $creditE = array(
                    'voucher_date' => $request->invoice_date,
                    'voucher_number' => $request->invoice_number,
                    'general_ledger_account_id' => $customer->general_ledger_account_id,
                    'note' => $request->note,
                    'debit' => 0,
                    'credit' => $request->net_total,
                    'branch' => Auth::user()->branch,
                    'created_at' => date('Y-m-d H:i:s'),
                );
                $this->insertDoubleEntry($creditE);
            }
            $credit = array(
                'voucher_date' => $request->invoice_date,
                'voucher_number' => $request->invoice_number,
                'general_ledger_account_id' => Config::get('constants.SALE_ACCOUNT_GENERAL_LEDGER'),
                'note' => $request->note,
                'debit' => 0,
                'credit' => $request->net_total,
                'branch' => Auth::user()->branch,
                'created_at' => date('Y-m-d H:i:s'),
            );
            $this->insertDoubleEntry($credit);
            /**
             * Insert Stock Entry for each time
             * 1.get category of item
             *2.get linked general ledger account id from category table
             * Stock Amount will be reduced according to purchase price of the item
             */
            foreach ($items_detail as $_detail) {
                $item = DB::table('items')->where('id', $_detail['item_id'])->first();
                $category = DB::table('category')->where('id', $item->category)->first();
                $company = DB::table('companyinfo')->first();
                if ($company->stock_calculation == 0) {
                    $credit = array(
                        'voucher_date' => $request->invoice_date,
                        'voucher_number' => $request->invoice_number,
                        'general_ledger_account_id' => $category->general_ledger_account_id,
                        'note' => $item->name . ' ' . $_detail['item_qty'] . ' @ ' . $_detail['item_price'],
                        'debit' => 0,
                        'credit' => $_detail['amount'],
                        'branch' => Auth::user()->branch,
                        'created_at' => date('Y-m-d H:i:s'),
                    );
                    $this->insertDoubleEntry($credit);
                }
                $record = DB::table('items')->where('id', $_detail['item_id'])->first();
                $unseri = unserialize($record->linked_items);
                if (!empty($unseri)) {

                    if (count($unseri) > 0) {
                        foreach ($unseri as $value) {
                            $qty = $value['item_qty'] * $_detail['item_qty'];
                            $stock  = array(
                                'voucher_date' => $request->invoice_date,
                                'voucher_number' => $request->invoice_number,
                                'transaction_type' => '-',
                                'general_ledger_account_id' => $category->general_ledger_account_id,
                                'item_qty' => $qty,
                                'item_id' => $value['item_id'],
                                'branch' => Auth::user()->branch,
                                'created_at' => date('Y-m-d H:i:s'),
                            );
                            $this->stockManagementEntry($stock);
                        }
                    }
                }
                // else {

                $stock  = array(
                    'voucher_date' => $request->invoice_date,
                    'voucher_number' => $request->invoice_number,
                    'transaction_type' => '-',
                    'general_ledger_account_id' => $category->general_ledger_account_id,
                    'item_qty' => $_detail['item_qty'],
                    'item_id' => $_detail['item_id'],
                    'branch' => Auth::user()->branch,
                    'created_at' => date('Y-m-d H:i:s'),
                );
                $this->stockManagementEntry($stock);
                // }
            }
            $sale = array(
                'net_total' => $request->net_total,
                'gross_amount' => $request->gross_amount,
                'discount_amount' => $request->discount_amount,
                'created_at' => date('Y-m-d H:i:s'),
                'customer_id' => $request->customer_id,
                'items_detail' => serialize($items_detail),
                'invoice_number' => $request->invoice_number,
                'note' => $request->note,
                'net_pcs' => $request->net_pcs,
                'net_qty' => $request->net_qty,
                'invoice_date' => $request->invoice_date,
                'net_custom_rate' => $request->net_custom_rate,
                'cash_entry' => $request->cash_entry ?? 0,
                'branch' => Auth::user()->branch,
            );
            DB::table('sales')->insert($sale);
            /***
             * add entry to transaction log
             *
             */
            $log = array(
                'user_id' => Auth::user()->id,
                'voucher_number' => $request->invoice_number,
                'transaction_action' => 'Add',
                'transaction_detail' => serialize($sale),
                'branch' => Auth::user()->branch,
                'transaction_type' => 'Sale Invoice',
                'created_at' => date('Y-m-d H:i:s'),
            );
            $this->addTransactionLog($log);

            return response()->json(['success' => true, 'message' => 'Sale Invoice added successfully..', 'redirectUrl' => '/sales/list'], 200);
        }
    }
    public function edit($id)
    {
        $sale = DB::table('sales')->where('id', $id)->first();
        $customers = DB::table('customers')->where('status', 1)->where('branch', Auth::user()->branch)->get();
        $items = DB::table('items')->where('branch', Auth()->user()->branch)->get();
        return view('sales.new', array('sale' => $sale, 'customers' => $customers, 'items' => $items));
    }
    public function update(Request $request)
    {
        $response = array('success' => false, 'message' => '', 'redirectUrl' => '');
        $validator = Validator::make(
            $request->all(),
            [
                'invoice_number' => 'required',
                'invoice_date' => 'required',
                'customer_id' => 'required|numeric',
                'net_total' => 'required|numeric|min:0|not_in:0',
                'note' => 'required',
                'net_pcs' => 'required|numeric|min:0|not_in:0',
                'net_qty' => 'required|numeric|min:0|not_in:0',
            ],
            [
                'invoice_number.required' => 'The Invoice #  is required.',
                'invoice_date.required' => 'The Invoice Date  is required.',
                'customer_id.required' => 'The Customer   is required.',
                'net_total.required' => 'Net Total   is required.',
                'note.required' => 'Notes   is required.',
                'total_pcs.required' => 'Pcs is required.',
            ]
        );
        if ($validator->fails()) {
            //$response['message'] = $validator->messages();
            return response()->json(array(
                'success' => false,
                'errors' => $validator->getMessageBag()->toArray()

            ), 422);
        } else {

            $items_detail = array();
            $item_ids = $request->item_id;
            $item_prices = $request->item_price;
            $item_qtys = $request->item_qty;
            $item_pcss = $request->item_pcs;
            $item_custom_rates = $request->item_custom_rate;
            $total_custom_rates = $request->total_custom_rate;
            $amounts = $request->amount;
            $i = 0;
            foreach ($item_ids as $item) {
                $itemid = $item_ids[$i];
                $price = $item_prices[$i];
                $pcs = $item_pcss[$i];
                $qty = $item_qtys[$i];
                $item_custom_rate = $item_custom_rates[$i];
                $total_custom_rate = $total_custom_rates[$i];
                $amount = $amounts[$i];
                if ($amount > 0) {
                    $items_detail[] = array(
                        'item_id' => $itemid,
                        'item_price' => $price,
                        'item_qty' => $qty,
                        'amount' => $amount,
                        'item_custom_rate' => $item_custom_rate,
                        'total_custom_rate' => $total_custom_rate,
                        'item_pcs' => $pcs,
                    );
                }

                $i++;
            }


            /**Delete first all entries from General Ledger Transactions Table
             * Insert Double entry
             *Customer A/c Debit
             *Sale A/c  Credit
             */
            $this->deleteDoubleEntry($request->invoice_number);
            $this->stockManagementEntryDelete($request->invoice_number);
            $customer = DB::table('customers')->where('id', $request->customer_id)->first();
            $debit = array(
                'voucher_date' => $request->invoice_date,
                'voucher_number' => $request->invoice_number,
                'general_ledger_account_id' => $customer->general_ledger_account_id,
                'note' => $request->note,
                'debit' => $request->net_total,
                'credit' => 0,
                'branch' => Auth::user()->branch,
                'updated_at' => date('Y-m-d H:i:s'),
            );
            $this->insertDoubleEntry($debit);
            if(isset($request->cash_entry)){
                $creditE = array(
                    'voucher_date' => $request->invoice_date,
                    'voucher_number' => $request->invoice_number,
                    'general_ledger_account_id' => $customer->general_ledger_account_id,
                    'note' => $request->note,
                    'debit' => 0,
                    'credit' => $request->net_total,
                    'branch' => Auth::user()->branch,
                    'created_at' => date('Y-m-d H:i:s'),
                );
                $this->insertDoubleEntry($creditE);
            }
            $credit = array(
                'voucher_date' => $request->invoice_date,
                'voucher_number' => $request->invoice_number,
                'general_ledger_account_id' => Config::get('constants.SALE_ACCOUNT_GENERAL_LEDGER'),
                'note' => $request->note,
                'debit' => 0,
                'credit' => $request->net_total,
                'branch' => Auth::user()->branch,
                'updated_at' => date('Y-m-d H:i:s'),
            );
            $this->insertDoubleEntry($credit);
            /**
             * Insert Stock Entry for each time
             * 1.get category of item
             *2.get linked general ledger account id from category table
             */
            foreach ($items_detail as $_detail) {
                $item = DB::table('items')->where('id', $_detail['item_id'])->first();
                $category = DB::table('category')->where('id', $item->category)->first();
                $company = DB::table('companyinfo')->first();
                if ($company->stock_calculation == 0) {
                    $credit = array(
                        'voucher_date' => $request->invoice_date,
                        'voucher_number' => $request->invoice_number,
                        'general_ledger_account_id' => $category->general_ledger_account_id,
                        'note' => $item->name . ' ' . $_detail['item_qty'] . ' @ ' . $_detail['item_price'],
                        'debit' => 0,
                        'credit' => $_detail['amount'],
                        'branch' => Auth::user()->branch,
                        'updated_at' => date('Y-m-d H:i:s'),
                    );
                    $this->insertDoubleEntry($credit);
                }
                $record = DB::table('items')->where('id', $_detail['item_id'])->first();
                $unseri = unserialize($record->linked_items);
                if (!empty($unseri)) {

                    if (count($unseri) > 0) {
                        foreach ($unseri as $value) {
                            $qty = $value['item_qty'] * $_detail['item_qty'];
                            $stock  = array(
                                'voucher_date' => $request->invoice_date,
                                'voucher_number' => $request->invoice_number,
                                'transaction_type' => '-',
                                'general_ledger_account_id' => $category->general_ledger_account_id,
                                'item_qty' => $qty,
                                'item_id' => $value['item_id'],
                                'branch' => Auth::user()->branch,
                                'created_at' => date('Y-m-d H:i:s'),
                            );
                            $this->stockManagementEntry($stock);
                        }
                    }
                }
                $stock  = array(
                    'voucher_date' => $request->invoice_date,
                    'voucher_number' => $request->invoice_number,
                    'transaction_type' => '-',
                    'general_ledger_account_id' => $category->general_ledger_account_id,
                    'item_qty' => $_detail['item_qty'],
                    'item_id' => $_detail['item_id'],
                    'branch' => Auth::user()->branch,
                    'created_at' => date('Y-m-d H:i:s'),
                );
                $this->stockManagementEntry($stock);
            }
            $sale = array(
                'net_total' => $request->net_total,
                'gross_amount' => $request->gross_amount,
                'discount_amount' => $request->discount_amount,
                'net_pcs' => $request->net_pcs,
                'net_qty' => $request->net_qty,
                'updated_at' => date('Y-m-d H:i:s'),
                'customer_id' => $request->customer_id,
                'items_detail' => serialize($items_detail),
                'invoice_number' => $request->invoice_number,
                'note' => $request->note,
                'invoice_date' => $request->invoice_date,
                'net_custom_rate' => $request->net_custom_rate,
                'cash_entry' => $request->cash_entry ?? 0,
                'branch' => Auth::user()->branch,

            );
            DB::table('sales')->where('id', $request->id)->update($sale);
            $log = array(
                'user_id' => Auth::user()->id,
                'voucher_number' => $request->invoice_number,
                'transaction_action' => 'Update',
                'transaction_detail' => serialize($sale),
                'branch' => Auth::user()->branch,
                'transaction_type' => 'Sale Invoice',
                'created_at' => date('Y-m-d H:i:s'),
            );
            $this->addTransactionLog($log);
            return response()->json(['success' => true, 'message' => 'Sale Updated added successfully..', 'redirectUrl' => '/sales/list'], 200);
        }
    }

    public function delete($id)
    {
        $sale = DB::table('sales')->where('id', $id)->first();
        $response = array('success' => false, 'message' => '', 'redirectUrl' => '');
        $this->deleteDoubleEntry($sale->invoice_number);
        DB::table('sales')->where('invoice_number', $sale->invoice_number)->delete();
        $log = array(
            'user_id' => Auth::user()->id,
            'voucher_number' => $sale->invoice_number,
            'transaction_action' => 'Delete',
            'transaction_detail' => serialize($sale),
            'branch' => Auth::user()->branch,
            'transaction_type' => 'Sale Invoice',
            'created_at' => date('Y-m-d H:i:s'),
        );
        $this->addTransactionLog($log);
        $this->stockManagementEntryDelete($sale->invoice_number);
        return response()->json(['success' => true, 'message' => 'Sale deleted successfully..', 'redirectUrl' => '/sales/list'], 200);
    }


    // Search Sales
    public function searchSales(Request $request)
    {
        $Queries = array();
        // if ($request->isMethod('GET')) {

        //     $request->from_date= $request->get('from_date');
        //     $request->to_date=$request->get('to_date');
        //     $request->invoice_number=$request->get('invoice_number');
        //     $request->customer_id=$request->get('customer_id');
        // }

        if(empty($request->from_date) && empty($request->to_date) &&  empty($request->customer_id) && empty($request->invoice_number)){
            return redirect('sales/list');
        }
        $query = DB::table('sales');
        $query->join('customers', 'sales.customer_id', '=', 'customers.id');
        $query->select('sales.*', 'customers.name as customer_name', 'customers.*');

        if(!empty($request->from_date) && !empty($request->to_date)){
            $Queries['from_date'] = $request->from_date;
            $Queries['to_date'] = $request->to_date;
            $query->whereBetween('sales.invoice_date', [$request->from_date, $request->to_date]);
        }
        if (!empty($request->invoice_number)) {
            $Queries['invoice_number'] = $request->invoice_number;
            $query->where('sales.invoice_number', 'like', "%$request->invoice_number%");
        }
        if (!empty($request->customer_id)) {
            $Queries['customer_id'] = $request->customer_id;
            $query->where('sales.customer_id', '=', $request->customer_id);
        }
        $list = $query->orderByDesc('sales.id')->paginate(20);
        $list->appends($Queries);
        $net_total = $query->sum('net_total');
        $net_qty = $query->sum('net_qty');
        $net_pcs = $query->sum('net_pcs');
        $customers = DB::table('customers')->get();
        return view('sales.list', array('salelist' => $list, 'from_date' => $request->from_date, 'to_date' => $request->to_date, 'customer_id' => $request->customer_id, 'invoice_number' => $request->invoice_number,'customers'=>$customers,'net_total'=>$net_total,'net_pcs'=>$net_pcs,'net_qty'=>$net_qty));
    }



    // PDF generator
    public function recordPDF($id)
    {
        $sale = DB::table('sales')->where('sales.id', $id)
            ->leftJoin('customers', 'sales.customer_id', '=', 'customers.id')
            ->first();
        $items = DB::table('items')->where('branch', Auth()->user()->branch)->get();
        $companyinfo = DB::table('companyinfo')->first();
        $companyinfo->logo = url('/') . $companyinfo->logo;

        $data =  array('sale' => $sale, 'items' => $items, 'companyinfo' => $companyinfo);
        $pdf = PDF::loadView('sales.salePdf', $data);
        // return $pdf->download('salePdf.pdf');
        return $pdf->stream('salePdf.pdf');
    }

    public function salePagePDF($from_date, $to_date, $customer_id, $invoice_number)
    {
        $query = DB::table('sales')
            ->join('customers', 'sales.customer_id', '=', 'customers.id')
            ->select('sales.*', 'customers.name as customer_name')->where('sales.branch', Auth::user()->branch);
        if ($from_date != 'none' && $to_date != 'none') {
            $query->whereBetween('sales.invoice_date', [$from_date, $to_date]);
        }
        if ($invoice_number != 'none') {
            $query->where('sales.invoice_number', 'like', "%$invoice_number%");
        }
        if ($customer_id != 'none') {
            $query->where('customers.id', "$customer_id");
        }

        $list = $query->orderByDesc('sales.id')->get();
        $net = $query->orderByDesc('sales.id')->sum('net_total');
        $companyinfo = DB::table('companyinfo')->first();
        $companyinfo->logo = url('/') . $companyinfo->logo;
        $data = array(
            'salelist' => $list,
            'companyinfo' => $companyinfo,
            'net' => $net
        );

        $pdf = PDF::loadView('sales.salePagePdf', $data);
        return $pdf->stream('pagePdf.pdf');
    }

    public function customerledger()
    {
        /* $response = array('success' => false, 'message' => '', 'redirectUrl' => '');
        DB::table('sales')->where('id',$id)->delete();
        return response()->json(['success' => true, 'message' => 'Sale deleted successfully..', 'redirectUrl' => '/sales/list'],200);*/
    }
    public function insertDoubleEntry($data)
    {
        /**
         * In case of exception,Roll Back whole Entry
         * remove double entry
         *
         */
        try {
            DB::table('general_ledger_transactions')->insertGetId($data);
        } catch (\Exception $e) {
            DB::table('general_ledger_transactions')->where('voucher_number', $data->voucher_number)->delete();
            return response()->json(['success' => false, 'message' => $e->getMessage(), 'redirectUrl' => '/sales/list'], 200);
        }
    }
    public function updateDoubleEntry($data)
    {
        /**
         * In case of exception,no need to
         * remove double entry while updated because of
         * record already exisit in table
         * no mettars if no updated
         */
        try {
            DB::table('general_ledger_transactions')
                ->where('voucher_number', $data['voucher_number'])
                ->where('general_ledger_account_id', $data['general_ledger_account_id'])
                ->update($data);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage(), 'redirectUrl' => '/sales/list'], 200);
        }
    }
    public function deleteDoubleEntry($voucher_number)
    {
        try {
            DB::table('general_ledger_transactions')->where('voucher_number', $voucher_number)->delete();
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage(), 'redirectUrl' => '/sales/list'], 200);
        }
    }


    public function stockManagementEntry($data)
    {
        try {
            DB::table('general_inventory_transactions')->insertGetId($data);
        } catch (\Exception $e) {
            DB::table('general_inventory_transactions')->where('voucher_number', $data->voucher_number)->delete();
            return response()->json(['success' => false, 'message' => $e->getMessage(), 'redirectUrl' => '/sales/list'], 200);
        }
    }

    public function addTransactionLog($data)
    {
        DB::table('transactions_log')->insertGetId($data);
    }

    public function stockManagementEntryDelete($voucher_number)
    {
        try {
            DB::table('general_inventory_transactions')->where('voucher_number', $voucher_number)->delete();
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage(), 'redirectUrl' => '/sales/list'], 200);
        }
    }
}
